"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tracker = void 0;
const lodash_clonedeep_1 = __importDefault(require("lodash.clonedeep"));
const constants_1 = require("./constants");
class Tracker {
    constructor(trackerConfig) {
        this.history = Object.fromEntries(constants_1.queryMethods.map((method) => [method, []]));
        this.on = Object.fromEntries(constants_1.queryMethods.map((method) => [method, this.prepareStatement(method)]));
        this.responses = new Map();
        this.config = trackerConfig;
        this.reset();
    }
    _handle(rawQuery) {
        return new Promise((resolve, reject) => {
            setTimeout(async () => {
                if (typeof rawQuery.method === 'undefined' &&
                    constants_1.transactionCommands.some((trxCommand) => rawQuery.sql.startsWith(trxCommand))) {
                    return resolve(undefined);
                }
                const possiableMethods = [rawQuery.method, 'any'];
                for (const method of possiableMethods) {
                    const handlers = this.responses.get(method) || [];
                    for (let i = 0; i < handlers.length; i++) {
                        const handler = handlers[i];
                        if (handler.match(rawQuery)) {
                            this.history[method].push(rawQuery);
                            if (handler.errorMessage) {
                                reject(new Error(handler.errorMessage));
                            }
                            else {
                                const data = typeof handler.data === 'function' ? await handler.data(rawQuery) : handler.data;
                                resolve((0, lodash_clonedeep_1.default)(Tracker.applyPostOp(data, rawQuery)));
                            }
                            if (handler.once) {
                                handlers.splice(i, 1);
                            }
                            return;
                        }
                    }
                }
                reject(new Error(`Mock handler not found`));
            }, 0);
        });
    }
    reset() {
        this.resetHandlers();
        this.resetHistory();
    }
    resetHandlers() {
        constants_1.queryMethods.forEach((method) => this.responses.set(method, []));
    }
    resetHistory() {
        constants_1.queryMethods.forEach((method) => (this.history[method].length = 0));
    }
    prepareMatcher(rawQueryMatcher) {
        if (typeof rawQueryMatcher === 'string' && rawQueryMatcher) {
            return (rawQuery) => rawQuery.sql.includes(rawQueryMatcher);
        }
        else if (rawQueryMatcher instanceof RegExp) {
            return (rawQuery) => rawQueryMatcher.test(rawQuery.sql);
        }
        else if (typeof rawQueryMatcher === 'function') {
            return rawQueryMatcher;
        }
        throw new Error('Given invalid query matcher');
    }
    prepareStatement(queryMethod) {
        return (rawQueryMatcher) => {
            const matcher = this.prepareMatcher(rawQueryMatcher);
            return {
                response: (data) => {
                    const handlers = this.responses.get(queryMethod) || [];
                    handlers.push({ match: matcher, data });
                    return this;
                },
                responseOnce: (data) => {
                    const handlers = this.responses.get(queryMethod) || [];
                    handlers.push({ match: matcher, data, once: true });
                    return this;
                },
                simulateError: (errorMessage) => {
                    const handlers = this.responses.get(queryMethod) || [];
                    handlers.push({ match: matcher, data: null, errorMessage });
                    return this;
                },
                simulateErrorOnce: (errorMessage) => {
                    const handlers = this.responses.get(queryMethod) || [];
                    handlers.push({ match: matcher, data: null, once: true, errorMessage });
                    return this;
                },
            };
        };
    }
    static applyPostOp(data, rawQuery) {
        if (rawQuery.postOp === 'first' && Array.isArray(data)) {
            return data[0];
        }
        else if (rawQuery.postOp === 'pluck' && rawQuery.pluck && Array.isArray(data)) {
            return data.map((item) => item[rawQuery.pluck]);
        }
        return data;
    }
}
exports.Tracker = Tracker;
